# -*- coding: utf-8 -*-

"""
Colormaps
=========

The `colormap` module contains definition of common colormaps and tools
to manipulate and create them.

.. automodule:: plotpy.widgets.colormap.widget
    :members:
.. automodule:: plotpy.widgets.colormap.editor
    :members:
.. automodule:: plotpy.widgets.colormap.manager
    :members:
"""
